import tkinter as tk
import tkinter.ttk as tkk
from tk_html_widgets import HTMLLabel
from PIL import ImageTk, Image
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg

from VisualizePloter import VisualizeFacade


class Frame(tk.Frame):
    def __init__(self, parent):
        tk.Frame.__init__(self, parent)
        self._parent = parent
        for i in range(5):
            self.grid_rowconfigure(i, weight=1)
            self.grid_columnconfigure(i, weight=1)

    @property
    def button_width(self):
        return int(self._parent.winfo_screenwidth() * 0.01)

    @property
    def button_height(self):
        return int(self._parent.winfo_screenheight() * 0.002)

    def createBottom(self, text, function, row, column):
        button = tk.Button(self, text=text, width=self.button_width,
                           height=self.button_height,
                           command=function)
        button.grid(row=row, column=column, pady=10)

    def createLabel(self, text, front, row, column):
        label = tk.Label(self, text=text, font=("Arial", front))
        label.grid(row=row, column=column, columnspan=3, pady=10)

    def createImage(self, path, row, column):
        img = Image.open(path)
        width, height = img.size
        img = img.resize((width // 2, height // 2))
        photo = ImageTk.PhotoImage(img)

        label = tk.Label(self, image=photo)
        label.image = photo
        label.grid(row=row, column=column, padx=10, pady=10)

    def createCombobox(self, choices, function):
        combobox = tkk.Combobox(self, values=choices)
        combobox.bind("<<ComboboxSelected>>", function)
        return combobox


class HomePage(Frame):
    def __init__(self, parent):
        super().__init__(parent)
        self.__menu = MainMenuPage(parent, self)

        self.createLabel("Moody Blues", 50, 0, 2)
        self.createImage("Image/GraduationCaps.png", 1, 2)
        self.createBottom("Start", self.switchToDataMenu, 2, 2)
        self.createBottom("Exit", parent.destroy, 3, 2)

    def display(self):
        self.grid(sticky="nsew")

    def switchToDataMenu(self):
        self.grid_forget()
        self.__menu.display()


class MainMenuPage(Frame):
    def __init__(self, parent, home):
        super().__init__(parent)

        self.__home = home
        self.__dataPage = DetailPage(parent)
        self.__virtualize = VisualizePage(parent)

        self.__dataPage.grid(row=0, column=0)
        self.__virtualize.grid(row=0, column=0)

        self.createBottom("About this program", self.detailScreen, 4, 0)
        self.createBottom("Data Visualize", self.dataScreen, 4, 2)
        self.createBottom("Back to home", self.switchToMainMenu, 4, 4)

    def display(self):
        self.detailScreen()
        self.grid(sticky="nsew")

    def switchToMainMenu(self):
        self.grid_forget()
        self.__virtualize.grid_forget()
        self.__dataPage.grid_forget()
        self.__home.grid(sticky="nsew")

    def detailScreen(self):
        self.__dataPage.grid(row=0, column=0)
        self.__virtualize.disappear()
        self.__dataPage.display()

    def dataScreen(self):
        self.__virtualize.grid(row=0, column=0)
        self.__dataPage.disappear()
        self.__virtualize.display()


class DetailPage(Frame):
    def __init__(self, parent):
        super().__init__(parent)
        self.__facade = VisualizeFacade()

    def display(self):
        self.createLabel("About Moody Blues", 40, 0, 0)

        html = self.__facade.getHTML()
        html_widget = HTMLLabel(self, html=html, borderwidth=1, relief='solid')
        scrollbar = tk.Scrollbar(self, orient='vertical',
                                 command=html_widget.yview)
        html_widget.configure(yscrollcommand=scrollbar.set)
        html_widget.grid(row=1, column=1, rowspan=3, columnspan=2,
                         sticky='nsew')
        scrollbar.grid(row=1, column=4, rowspan=4, sticky='ns')

        self.grid(sticky="nsew")

    def disappear(self):
        self.grid_forget()


class VisualizePage(Frame):
    def __init__(self, parent):
        super().__init__(parent)

        self.__canvas = None
        self.__facade = VisualizeFacade()

        values = self.__facade.getChoices("values")
        self.__box = [self.createCombobox(values, self.update_combobox),
                      self.createCombobox('', self.update_combobox),
                      self.createCombobox('', self.plot),
                      self.createCombobox('', self.draw)]

    def display(self):
        self.createLabel("Visualize", 40, 0, 1)

        self.comboDisplay(0, 0, 'Gender')
        self.comboDisplay(1, 1, 'Mental health')
        self.comboDisplay(2, 3, 'bar')
        self.comboDisplay(3, 4, 'Year of study')

        self.grid(sticky="nsew")

    def comboDisplay(self, index, column, text):
        self.__box[index].grid(row=1, column=column, padx=10, pady=10)
        self.__box[index].set(text)
        self.__box[index].event_generate("<<ComboboxSelected>>")

    def update_combobox(self, event):
        choice = self.__facade.getChoices(event.widget.get())
        if event.widget == self.__box[0]:
            self.__box[1].config(values=choice)
            self.__box[1].set('2nd data')

        alt = self.__box[0].get()
        self.__box[2].set('Graph')
        self.__box[2].config(values=self.__facade.getChoices("Graph", alt))

        if event.widget == self.__box[1] and event.widget.get() != 'None':
            self.__box[2].config(values=self.__facade.getChoices("Relationship"
                                                                 , alt))
            self.__box[2].set('Relationship')

        self.__box[3].set('')
        self.__box[3].config(values=[''])

    def plot(self, event):
        alternative = self.__box[1].get()
        values = self.__facade.getChoices(self.__box[0].get(), alternative)
        values2 = self.__facade.getChoices("layout")
        self.draw('')
        if "network" in event.widget.get():
            self.__box[3].set('Layout')
            self.__box[3].config(values=values2)
        elif self.__box[3].get() in ['', 'Layout'] + values2 and \
                alternative not in ['2nd data', 'None']:
            self.__box[3].set('Hue')
            self.__box[3].config(values=values)

    def draw(self, event):
        graph = self.__facade.plot_data(*[i.get() for i in self.__box])
        if graph:
            if self.__canvas:
                self.__canvas.get_tk_widget().delete('all')
            self.__canvas = FigureCanvasTkAgg(graph, master=self)
            self.__canvas.draw()
            self.__canvas.get_tk_widget().grid(row=3, column=0, pady=25,
                                               rowspan=2, columnspan=5)

    def disappear(self):
        self.grid_forget()
