import math

import networkx as nx
import seaborn as sns
from matplotlib import pyplot as plt

from DataCollector import DataFrameModifier, HTMLReader


class VisualizeFacade:
    _instance = None

    def __new__(cls):
        if cls._instance is None:
            cls._instance = super().__new__(cls)
            cls._instance._initialized = False
        return cls._instance

    def __init__(self):
        if self._initialized:
            return
        self._initialized = True
        self.htmlText = HTMLReader()
        self.dfModify = DataFrameModifier()
        self.graph = GraphMaker()
        self.network = NetworkGraph()
        self.graphChoice = ['bar', 'pie', 'count', 'box', 'histogram']
        self.relationChoice = ['bar', 'box', 'violin', 'scatter',
                               'network relationship']
        self.layout = ['spring', 'circular']
        self.denied = [None, '', "None", '2nd data', 'Hue', 'Layout']

    def getChoices(self, choice, alternative=None):
        values = list(self.dfModify.normal.columns)

        match choice:
            case "values":
                return values
            case "Graph":
                values = self.graphChoice.copy()
                if alternative:
                    if self.dfModify.noText[alternative].dtype == "object":
                        values.remove('histogram')
                        values.remove('bar')
                        values.remove('box')
                return values
            case "Relationship":
                values = self.relationChoice.copy()
                if alternative:
                    if alternative in ['Gender', "Marital Status",
                                       "Depression", "Anxiety", "Panic attack",
                                       "Treatment"]:
                        values.remove('network relationship')
                return values
            case "layout":
                return self.layout

        if alternative in values:
            values.remove(alternative)
        if choice != "None":
            values.remove(choice)
        if choice == "Course":
            values.remove("Gender")
        else:
            values.remove("Course")
        values.append('None')
        return values

    def plot_data(self, x_column=None, y_column=None, genre=None,
                  hue_column=None):
        plt.close('all')
        kwargs = {"data": self.dfModify.normal, "x": x_column, "kind": genre}
        label = ""

        if y_column not in self.denied:
            kwargs["y"] = y_column
            label += f" by {y_column}"
            if kwargs["data"][x_column].dtype == "object" \
                    and kwargs["data"][y_column].dtype == "object":
                kwargs["data"] = self.dfModify.noText

        if hue_column not in self.denied:
            kwargs["hue"] = hue_column
            label += f" grop by {hue_column}"

        if genre == "network relationship":
            kwargs.pop("kind")
            kwargs["data"] = self.dfModify.text
            return self.network.relationship(**kwargs)

        if hue_column in self.layout:
            kwargs.pop("hue")

        if genre == "pie":
            return self.graph.pie(x_column, self.dfModify.counts(x_column))
        elif genre == "histogram":
            return self.graph.histogram(x_column,
                                        self.dfModify.noText[x_column])
        elif genre == "scatter":
            if x_column == "Course":
                kwargs["x"], kwargs["y"] = kwargs["y"], kwargs["x"]
            return self.graph.scatter(self.denied, **kwargs)
        elif genre == "bar":
            kwargs["data"] = self.dfModify.noText
            kwargs["errorbar"] = None
        return self.graph.default(label, **kwargs)

    def getHTML(self):
        text = self.htmlText.getBody()
        return text


class GraphMaker:
    @staticmethod
    def default(label, **kwargs):
        plot = sns.catplot(**kwargs, height=5, aspect=1.5)
        title = f"{kwargs['kind']} plot of {kwargs['x']}" + label
        plot.fig.suptitle(title)
        return plot.fig

    @staticmethod
    def histogram(x, data):
        scale = 1
        if data.max() - data.min() <= 5:
            scale = 10
        bins = [i / scale for i in range(int(data.min()) * scale,
                                         int(data.max() * scale + 1),
                                         math.ceil(scale / 2))]
        fig, ax = plt.subplots(figsize=(7.5, 5))
        ax.hist(data, bins=bins, edgecolor='black')
        ax.set_title(f"Histogram of {x}")
        return fig

    @staticmethod
    def pie(x, data):
        labels = data.index
        if data[0] != "string":
            labels = sorted(labels)
        fig, ax = plt.subplots(figsize=(7.5, 5))
        ax.pie(data, startangle=90, labels=labels, autopct='%1.1f%%')
        ax.axis('equal')
        ax.set_title(f"Percentage of Students in Each {x}")
        return fig

    @staticmethod
    def scatter(denied, **kwargs):
        plot = sns.relplot(**kwargs, height=5, aspect=1.5)
        label = ""
        if kwargs.get("hue") is not None:
            if kwargs['hue'] not in denied:
                label = f" color by {kwargs['hue']}"
        title = f"Scatter plot between {kwargs['x']} and {kwargs['y']}" + label
        plot.fig.suptitle(title)
        return plot.figure


class NetworkGraph:
    def __init__(self):
        self.__graph = None

    def relationship(self, data=None, x=None, y=None, hue='circular'):
        self.__graph = nx.Graph()
        nodes = data[x].unique()
        for node in nodes:
            self.__graph.add_node(node)
        for i in range(len(data)):
            for j in range(i + 1, len(data)):
                if data[y][i] == data[y][j] and data[x][i] != data[x][j]:
                    if not self.__graph.has_edge(data[x][i], data[x][j]):
                        self.__graph.add_edge(data[x][i], data[x][j], weight=1)
                    else:
                        self.__graph[data[x][i]][data[x][j]]['weight'] += 1

        pos = self.__getLayout(hue)
        fig, ax = plt.subplots(figsize=(7.5, 5))
        nx.draw(self.__graph, pos, ax=ax, with_labels=True, font_size=8)
        if x != "Course":
            labels = nx.get_edge_attributes(self.__graph, 'weight')
            nx.draw_networkx_edge_labels(self.__graph, pos, ax=ax, font_size=6,
                                         edge_labels=labels)

        ax.set_axis_off()
        plt.title(f"Relation between each {x} by {y} network graph")
        plt.tight_layout()
        return fig

    def __getLayout(self, layout):
        if layout == 'spring':
            return nx.spring_layout(self.__graph)
        elif layout == 'spectral':
            return nx.spectral_layout(self.__graph)
        else:
            return nx.circular_layout(self.__graph)
